Documentation
¶
Index ¶
Constants ¶
This section is empty.
Variables ¶
Available loggers
Functions ¶
func IfTracing ¶
IfTracing will call the given function with Trace logger if current logging level is to enable traces Allows to create time-consuming traces only when they are traced.
func SetLogLevel ¶
func SetLogLevel(lvl Level)
SetLogLevel sets the current logging level to given one
Types ¶
Click to show internal directories.
Click to hide internal directories.