Documentation
¶
Index ¶
Constants ¶
const ( UUID_LENGTH int = 16 UUID_STR_LEN int = 36 UUID16_LENGTH int = 2 UUID16_STR_LEN int = UUID16_LENGTH * 2 UUID32_LENGTH int = 4 UUID32_STR_LEN int = UUID32_LENGTH * 2 )
Variables ¶
var ErrData = errors.New("invalid data length for UUID")
error returned if UUID can not be parsed from given data
var ErrString = errors.New("can not parse UUID from input")
error returned if UUID can not be parsed from string
Functions ¶
This section is empty.
Types ¶
type Uuid ¶
type Uuid [16]byte
Bluetooth Core 5.2 Vol 3, part B, ch 2.5.1
var NoneUuid Uuid
Empty UUID value
type Uuid16 ¶
type Uuid16 uint16
func Uuid16FromBytes ¶
Parse UUID16 from byte slice. Error is returned if slice does not contain enough data
func Uuid16FromString ¶
Parse UUID16 from string. Error is returned if string is malformed
type Uuid32 ¶
type Uuid32 uint32
func Uuid32FromBytes ¶
Parse UUID32 from byte slice. Error is returned if slice does not contain enough data
func Uuid32FromString ¶
Parse UUID32 from string. Error is returned if string is malformed
Click to show internal directories.
Click to hide internal directories.